using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;

namespace voql
{
    public class ControlPositioning
    {
        /// <summary>
        /// Saves the position and size of the given control.
        /// If control is a Form object it saves its window position.
        /// Best placed in the OnClosing method of the (parent) Form.
        /// </summary>
        /// <param name="control">The control to be saved</param>
        public static void Save(Control control, System.Collections.Specialized.NameValueCollection values)
        {

			if( control == null )
            {
				throw new ArgumentException("The control cannot be null");
			}

			// form state
			Form form = control as Form;
			if( form != null )
            {
				values.Set(form.Name + "_windowState", Convert.ToString((int)form.WindowState));
				// if window is not in normal mode set to normal before saving
				if(form.WindowState != FormWindowState.Normal)
                {
					// save normal windo size
					form.WindowState = FormWindowState.Normal;
				}
			}

			// save screen res
			values.Set(control.Name + "_screenHeight", Convert.ToString(Screen.PrimaryScreen.Bounds.Height));
            values.Set(control.Name + "_screenWidth", Convert.ToString(Screen.PrimaryScreen.Bounds.Width));

			

			// control position/size
            values.Set(control.Name + "_top", Convert.ToString(control.Top));
            values.Set(control.Name + "_left", Convert.ToString(control.Left));
            values.Set(control.Name + "_width", Convert.ToString(control.Width));
            values.Set(control.Name + "_height", Convert.ToString(control.Height));

		}


        static private int GetNameValue(string key, System.Collections.Specialized.NameValueCollection values, int defaultValue)
        {
            int retValue = defaultValue;

            object objValue = values.Get(key);
            if (objValue != null)
            {
                try
                {
                    retValue = Convert.ToInt32(objValue as string);
                }
                catch (FormatException)
                {
                }
            }
            
            return retValue;
        }

		/// <summary>
		/// Loads the position and size of a control.
		/// If control is a form also loads the window position.
		/// Best placed in the Form's constructor.
		/// </summary>
		/// <param name="control">The control to be loaded.</param>
		public static void Load(Control control, System.Collections.Specialized.NameValueCollection values)
        {

			if( control == null )
            {
				throw new ArgumentException("The control cannot be null");
			}

            // check that screen res is the same
			// if not or can't be found revert to origional.
            try
            {
                if (Screen.PrimaryScreen.Bounds.Height != GetNameValue(control.Name + "_screenHeight", values, Screen.PrimaryScreen.Bounds.Height) ||
                    Screen.PrimaryScreen.Bounds.Width != GetNameValue(control.Name + "_screenWidth", values, Screen.PrimaryScreen.Bounds.Width))
                {

					return;
				}
			}
			catch (NullReferenceException)
            {
                return;
            }

			control.SuspendLayout();

			// form state
			Form form = control as Form;
			if( form != null )
            {

				try
                {
                    form.WindowState = (FormWindowState)GetNameValue(form.Name + "_windowState", values, (int)form.WindowState);
					form.StartPosition = FormStartPosition.Manual;
				}
				catch (NullReferenceException)
                {
                }
			}

			// control position/size
			try
            {
                control.Top = GetNameValue(control.Name + "_top", values, control.Top);
			}
			catch (NullReferenceException)
            {
            }
			try
            {
                control.Left = GetNameValue(control.Name + "_left", values, control.Left);
			}
			catch(NullReferenceException)
            {
            }
			try
            {
                control.Width = GetNameValue(control.Name + "_width", values, control.Width);
			}
			catch (NullReferenceException)
            {
            }
			try
            {
                control.Height = GetNameValue(control.Name + "_height", values, control.Height);
			}
			catch(NullReferenceException)
            {
            }

			control.ResumeLayout();

		}
    }
}
