using System;
using System.Reflection;
using System.Xml;
using ArrayList = System.Collections.ArrayList;
using System.Windows.Forms;
using System.Collections.Specialized;

using Telerik.OpenAccess;
using PersistenceCapable = Telerik.OpenAccess.SPI.dataobjects.PersistenceCapable;
using Telerik.OpenAccess.Debug;

namespace voql
{
    /// <summary>
    /// Summary description for OQLQueryBrowser.
    /// </summary>
    internal class OQLQueryBrowser : IDisposable
    {
        QueryProperties props;
        Database db;
        IObjectScope scope;
        Telerik.OpenAccess.Debug.ClassMetaData[] cmdArray;

        public OQLQueryBrowser(QueryProperties props)
        {
            this.props = props;
        }

        public void Dispose()
        {
            if (scope != null)
                scope.Dispose();
            if (db != null)
                db.Dispose();
        }

        private IObjectScope Scope
        {
            get
            {
                if (scope == null)
                {
                    Assembly entryAssembly = Assembly.LoadFrom(this.props.assemblyName);
                    string configFile =
                        "<openaccess>" +
                        "	<references>" +
                        "		<reference assemblyname=\"" + entryAssembly.FullName + "\" configrequired=\"true\"/>" +
                        "	</references>" +
                        "</openaccess>";

                    XmlDocument doc = new XmlDocument();
                    doc.InnerXml = configFile;
                    XmlElement xmlElement = (XmlElement)doc.FirstChild;

                    db = Database.Get(props.connectionId, xmlElement, new Assembly[] { entryAssembly });
                    scope = db.GetObjectScope();
                    cmdArray = Telerik.OpenAccess.Debug.ModelHelper.GetModel(scope);
                }
                return scope;
            }
        }

        public string GetSQL(string query)
        {
            string ret;
            try
            {
                ret = this.Scope.GetOqlQuery(query).BackendQuery;
            }
            catch (Exception e)
            {
                ret = PrintException(e, query);
            }
            return ret;
        }

        public string GetResult(string query, System.Windows.Forms.DataGrid grid)
        {
            string ret = GetSQL(query);
            try
            {
                IQueryResult qr = this.Scope.GetOqlQuery(query).Execute();
                if (qr.Count > 0)
                {
                    object first = qr[0];
                    PersistenceCapable pc = first as PersistenceCapable;
                    if (pc != null)
                    {
                        ObjectView ov = new ObjectView(this.Scope, qr, first.GetType());
                        ov.AllowListChanges = false;
                        ov.AllowUpdates = false;
                        grid.DataSource = ov;
                    }
                    else
                    {
                        ArrayList list = new ArrayList(qr.Count);
                        foreach (object o in qr)
                            list.Add(new Wrapper(o));
                        grid.DataSource = list;
                    }
                }
            }
            catch (Exception e)
            {
                ret = PrintException(e, query);
            }
            return ret;
        }

        private string PrintException(Exception e, string query)
        {
            string ret;

            OpenAccessException oe = e as OpenAccessException;

            if (oe != null)
            {
                if (e.InnerException != null)
                    ret = e.InnerException.Message + "\r\n" + e.InnerException.StackTrace;
                else
                {
                    ret = e.Message;
                    try
                    {
                        // Query Exception! Let's try to mark the wrong query string!
                        // first try:
                        // ["OrdersExtent",<42>,line=1,col=15]
                        int line;
                        int col;

                        string tmp = e.Message.Substring(e.Message.IndexOf("line=") + 5);
                        tmp = tmp.Substring(0, tmp.IndexOf(","));
                        line = Int32.Parse(tmp);

                        tmp = e.Message.Substring(e.Message.IndexOf("col=") + 4);
                        tmp = tmp.Substring(0, tmp.IndexOf("]"));
                        col = Int32.Parse(tmp);

                        string[] lines = query.Split('\n');
                        ret += "\r\n";
                        ret += lines[line - 1].Substring(0, col - 1);
                        ret += "~~~";
                        ret += lines[line - 1].Substring(col - 1);

                    }
                    catch (Exception)
                    {
                        try
                        {
                            // second try
                            // line 1:15: unexpected token: 
                            int line;
                            int col;

                            string tmp = e.Message.Substring(e.Message.IndexOf("line ") + 5);
                            tmp = tmp.Substring(0, tmp.IndexOf(":"));
                            line = Int32.Parse(tmp);

                            tmp = e.Message.Substring(e.Message.IndexOf("line ") + 5);
                            tmp = tmp.Substring(tmp.IndexOf(":") + 1);
                            tmp = tmp.Substring(0, tmp.IndexOf(":"));
                            col = Int32.Parse(tmp);

                            string[] lines = query.Split('\n');
                            ret += "\r\n";
                            ret += lines[line - 1].Substring(0, col - 1);
                            ret += "~~~";
                            ret += lines[line - 1].Substring(col - 1);

                        }
                        catch (Exception)
                        {
                            ;
                        }
                    }
                    if (oe.InnerExceptions != null)
                        foreach (Exception ei in oe.InnerExceptions)
                            ret += "\r\n\r\n" + PrintException(ei, query);
                }
            }
            else
            {
                if (e.InnerException != null)
                    ret = e.InnerException.Message + "\r\n" + e.InnerException.StackTrace;
                else
                {
                    ret = e.Message + "\r\n" + e.StackTrace;
                }
            }

            return ret;
        }

        public string FillTreeView(TreeView tree)
        {
            string ret = null;
            try
            {
                if (tree != null && tree.Nodes.Count == 0)
                {
                    IObjectScope scope = Scope;
                    if (cmdArray != null)
                    {
                        foreach (ClassMetaData cmd in cmdArray)
                        {
                            string classname = cmd.Name;
                            if (cmd.BaseType != null)
                                classname += " : " + cmd.BaseType.Name;
                            TreeNode node = new TreeNode(classname, 0, 0);
                            tree.Nodes.Add(node);
                            foreach (FieldMetaData fmd in cmd.GetFields())
                            {
                                string name = fmd.Name;
                                name += " - [" + fmd.FieldType.ToString() + "]";
                                node.Nodes.Add(new TreeNode(name, 1, 1));
                            }
                        }
                    }
                }
            }
            catch (Exception e)
            {
                cmdArray = null;
                ret = PrintException(e, "Try to fill persistent model tree.");
            }
            return ret;
        }

        // Appdomain entry point
        private static string libPath;
        private static string copyLibPath;
        private static Assembly mainAssembly;

        public static void StartQueryBrowserInAppdomain()
        {
            QueryProperties props = AppDomain.CurrentDomain.GetData("Properties") as QueryProperties;
            ResolveEventHandler resolveEventHandler = new ResolveEventHandler(GetAssembly);

            try
            {
                libPath = props.path;
                copyLibPath = props.copyFromPath;
                AppDomain.CurrentDomain.AssemblyResolve += resolveEventHandler;
                mainAssembly = Assembly.LoadFrom(props.assemblyName);
                OQLQueryBrowser browser = new OQLQueryBrowser(props);
                QueryDialog dlg = new QueryDialog();
                dlg.Browser = browser;
                dlg.Text = dlg.Text + " - " + props.shortName;
                dlg.ShowDialog();
            }
            catch (Exception e)
            {
                MessageBox.Show(e.Message, "OQL Query Browser");
            }
            finally
            {
                AppDomain.CurrentDomain.AssemblyResolve -= resolveEventHandler;
            }
        }

        public static Assembly GetAssembly(object sender, ResolveEventArgs args)
        {
            string searchName = args.Name;
            if (mainAssembly.FullName.Equals(searchName))
                return mainAssembly;
            int index = searchName.IndexOf(',');
            if (index > 0)
                searchName = searchName.Substring(0, index);

            //Copy any model referenced assemblies to temporary load folder
            System.IO.File.Copy(
                System.IO.Path.Combine(copyLibPath, searchName + ".dll"),
                libPath + "\\" + searchName + ".dll",
                true);
            //Load any model referenced assemblies from temporary load folder
            return Assembly.LoadFrom(libPath + "\\" + searchName + ".dll");
        }


        // start query browser in an new appdomain
        public static void StartQueryBrowser(string finalAssemblyName, string connectionId)
        {
            //1. Create temporary load folder 
            //2. Copy model assembly and any referenced assemblies to this temporary load folder
            //3. Load model assembly and any referenced assemblies from this temporary load folder
            string loadFolder = System.IO.Path.GetTempPath();
            if (!string.IsNullOrEmpty(loadFolder) && System.IO.Directory.Exists(loadFolder))
            {
                loadFolder = System.IO.Path.Combine(loadFolder, "voql\\");
            }
            else
            {
                loadFolder = "C:\\voql\\";
            }
            System.IO.Directory.CreateDirectory(loadFolder);

            // fill properties
            QueryProperties props = new QueryProperties();
            props.SetAssemblyNames(
                loadFolder + System.IO.Path.GetFileName(finalAssemblyName));
            props.connectionId = connectionId;
            props.copyFromPath = System.IO.Path.GetDirectoryName(finalAssemblyName);

            AppDomain domain = null;
            props.reload = true;
            props.modelVisible = false;
            props.controlsPositioning = new NameValueCollection();
            
            //Domain reload loop
            while (props.reload)
            {
                try
                {
                    props.reload = false;
                    //Copy model assembly to temporary load folder
                    System.IO.File.Copy(
                        finalAssemblyName,
                        loadFolder + System.IO.Path.GetFileName(finalAssemblyName), true);

                    // generate domain
                    AppDomainSetup domainSetup = new AppDomainSetup();
                    domainSetup.LoaderOptimization = System.LoaderOptimization.SingleDomain;
                    domainSetup.ConfigurationFile = System.AppDomain.CurrentDomain.SetupInformation.ConfigurationFile;

                    domain = AppDomain.CreateDomain("OQLQueryBrowser", null, domainSetup);
                    domain.SetData("Properties", props);

                    // start real tester in domain
                    domain.DoCallBack(new CrossAppDomainDelegate(StartQueryBrowserInAppdomain));

                    //Get possible changed properties values before the domain is unloaded.
                    props = domain.GetData("Properties") as QueryProperties;
                }
                finally
                {
                    if (domain != null)
                        AppDomain.Unload(domain);
                }
            }
        }
    }

    [Serializable]
    internal class QueryProperties
    {
        public string assemblyName;
        public string shortName;
        public string path;
        public string copyFromPath;
        public bool reload;
        public string oqlStatement;
        public bool modelVisible;
        public NameValueCollection controlsPositioning;

        public string connectionId;

        public void SetAssemblyNames(string fullname)
        {
            this.assemblyName = fullname;

            string tmp = fullname.Substring(fullname.LastIndexOf('\\') + 1);
            int lastDot = tmp.LastIndexOf('.');
            this.shortName = tmp.Substring(0, (lastDot < 0 ? tmp.Length : lastDot));
            this.path = fullname.Substring(0, fullname.LastIndexOf('\\'));
        }
    }

    internal class Wrapper
    {
        object o;

        public Wrapper(object o)
        {
            this.o = o;
        }

        public string Content
        {
            get
            {
                if (o == null) return "<null>";
                if (o.GetType() == typeof(object[]))
                {
                    string ret = "";
                    foreach (object o1 in (object[])o)
                        ret += o1.ToString() + " ";
                    return ret;
                }
                return o.ToString();
            }
        }
    }
}
